// Game.cpp: implementation of the CGame class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Game.h"
#include "StrategySpace.h"

#include <iostream.h>	//IO stream
#include <fstream.h>	//File stream

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGame::CGame(int seed, int wait, int iter, int N, int m, int q, int T, double tau, double phi, long *idum)
{
	int i;
	
	//Simulation length
	m_iSeed = seed;
	m_iWait = wait;
	m_iIter = iter;
	m_pidum = idum;

	m_iStep=0;
	m_iWin=-1;
	
	m_aOutput = new int*[m_iSeed+m_iWait+m_iIter];		//Output array
	for(i=0;i<m_iSeed+m_iWait+m_iIter;i++)
		m_aOutput[i] = new int[4];
	//Data stored in the output array is: winning option, volume, demand, simulated price

	m_iN = N;			//# of agents
	m_iM = m;
	m_iQ = q;
	m_iT = T;
	m_iTau = tau;
	m_iPhi = phi;

	//Create agents
	m_pAgents = new CAgent*[m_iN];

	//decide whether we create a strategy space
	if(m_iN*m_iQ > pow(2,pow(2,m_iM))) {
		cout << "Creating strategy space ..." << endl;
		m_pSSpace = new CStrategySpace(m_iM,m_iT,'F');
		for(i=0;i<m_iN;i++)
			m_pAgents[i] = new CAgent(m_iQ, m_iT, m_iTau, m_pSSpace, m_pidum);
		m_bSSpace = true;
	}	
	else {
		for(i=0;i<m_iN;i++)
			m_pAgents[i] = new CAgent(m_iQ, m_iM, m_iT, m_iTau, m_pidum);
		m_bSSpace = false;
	}


	//seed output matrix
	for (i=0;i<m_iSeed;i++)
		m_aOutput[i][0]=(int)(2*ran2(m_pidum));		//rounding takes place by simple truncation

	//check that seed is longer than any of the m values
	for (m_iStep=m_iM;m_iStep<m_iSeed;m_iStep++) {

		m_iWin=m_aOutput[m_iStep][0];

		//update virtual points
		if(m_bSSpace)
			m_pSSpace->points_update(hist(),m_iWin);
		else
			for(i=0;i<m_iN;i++)
				m_pAgents[i]->spoints_update(hist(), m_iWin);
	}


}

CGame::~CGame()
{
	int i;

	for(i=m_iSeed+m_iWait+m_iIter;i>0; --i)
		delete [] m_aOutput[i-1];
	delete [] m_aOutput;
	
	delete [] m_pAgents;	
}

//////////////////////////////////////////////////////////////////////
// Game
//////////////////////////////////////////////////////////////////////


//convert binary history of length m into a decimal
const int CGame::bhist2dec(int i, int m)
{
	int value=0;

	for (int j=0;j<m;j++)
		value += (int)(m_aOutput[i-j-1][0]*pow(2,j));	

	return value;
}

//return decimal representation of history bitstring of length m
int CGame::hist()
{
	if(m_Hist[0]==m_iStep)			//history stored is current
		return m_Hist[1];
	else {							//calculate history
		m_Hist[1] = bhist2dec(m_iStep,m_iM);
		m_Hist[0] = m_iStep;
		return m_Hist[1];
	}
}

//iterate the system
int CGame::go()
{
	int i;
	int D, V;
	int N0, N1;
	int a_pred;	//store the agent's predictions, -1, 0 or +1
	int mu;
	int L;

	for(m_iStep;m_iStep<(m_iSeed+m_iWait+m_iIter);m_iStep++)
	{
		N0 = 0;
		N1 = 0;
		mu = hist();		//global information

		//request an agent's prediction
		for(i=0;i<m_iN;i++) {
			a_pred = m_pAgents[i]->pred(mu);		//agent's make prediction
			if(a_pred==0)
				N0 ++;
			else if(a_pred==1)
				N1 ++;
		}
		D = N1 - N0;				//demand
		V = N0 + N1;				//volume
		L = (int)(m_iPhi * V);		//resource level

		//decide what the winning option is
		if(L == N1) 	//random coin flip
			m_iWin = (int)(2*ran2(m_pidum));
		else if(N1 < L)
			m_iWin = 1;
		else			
			m_iWin = 0;

		//store data
		m_aOutput[m_iStep][0] = m_iWin;		
		m_aOutput[m_iStep][1] = V;		
		m_aOutput[m_iStep][2] = D;		

		//update virtual points
		if(m_bSSpace)
			m_pSSpace->points_update(mu,m_iWin);
		else
			for(i=0;i<m_iN;i++)
				m_pAgents[i]->spoints_update(mu, m_iWin);		
	}

	//generate simulated price series:
	// the cumulative sum of the excess demand is the simplest way of doing this

	m_aOutput[m_iSeed+m_iWait][3] = 1000;		//initial price (arbitrary)
	for(i=m_iSeed+m_iWait;i<(m_iSeed+m_iWait+m_iIter-1);i++)
		m_aOutput[i+1][3] = priceupdate(m_aOutput[i][2],m_aOutput[i][3]);

	return 0;
}

//write out the game result as a datafile
int CGame::dataout(const char *OutFile)
{
	int i;

	//Open output stream
	ofstream OutStream(OutFile);
	if( !OutStream )
	{
		cerr << "Can't open " << OutFile << endl;
		return 1;
	}

	//....write out winning vector - removing seed and transient history
	for(i=m_iSeed+m_iWait;i<(m_iSeed+m_iWait+m_iIter);i++)
		OutStream << m_aOutput[i][0] << "\t" << m_aOutput[i][1] << "\t" << m_aOutput[i][2] << "\t" << m_aOutput[i][3] <<endl;

	return 0;
}




//generates an indicative price from the demand and the old price, we use the rule
//  p(t+1) = p(t)+d(t)
//i.e. if there is a positive demand for the asset, the price will rise and vice versa
int CGame::priceupdate(int demand, int price)
{
	return price + demand;	
}

// other possibilities include
//   p(t+1) = p(t) * exp(alpha * demand)
// where alpha is coefficient that determines how responsive the price is 
// to a change in the demand.  This form has the advantage that the price
// will never become negative.