#ifndef CGBINDING_H

#define CGBINDING_H

/********************************************************************************
** Include this header file into your application and inherit the base class
** CgBinding to provide a single common interface mechanism for loading Cg
** programs and performing run time binding of data assets.
**
** Written by John W. Ratcliff
** July 18, 2002
** Uses Doxygen style documentation header blocks.
*********************************************************************************/

#pragma warning( disable:4786 )  // warning C4503: '  ' : decorated name length exceeded, name was truncated

#include "cgbindingtype.h"

#include <vector>
#include <map>

class CgBinding;
class CgVertexShader;
class CgPixelShader;

/**
   @class CgVertexBind

   @brief A single bind point for a vertex CgProgram

   This class manages a single binding point between a particular instance of a
   Cg vertex program fragment and remaps it to a logical VertexBindingType.
   The application must be able to provide the data that is needed for this
   bind point.  For example the world*view*projection matrix.

   @see enum VertexBindingType

   @warning To minimize dependencies the internal bind point handle is case to a generic unsigned int

   @author John W. Ratcliff

   @date  July 18, 2002

*/
class CgVertexBind
{
public:
  CgVertexBind(void) { };

  CgVertexBind(VertexBindingType b,unsigned int handle)
  {
    mBind = b;
    mHandle = handle;
  };

  bool Activate(CgBinding *bindings,unsigned int program_handle,void *context1,void *context2);

private:
  VertexBindingType mBind;    /* Type of vertex binding operation */
  unsigned int      mHandle;  /* Handle to the Cg binding point iterator */
};

// STL vector of binding points
typedef std::vector< CgVertexBind > CgVertexBindVector;

/**
   @class CgVertexShader

   @brief A single instance of a vertexshader for a specific vertex format

   This class represents a single instance of a vertex program for a specific
   vertex format.

   @see enum  VertexBindingFormat
   @see class CgVertexBind

   @warning To minimize dependencies the internal Cg program handle is case to an unsigned int

   @author John W. Ratcliff

   @date  July 18, 2002

*/
class CgVertexShader
{
public:

  CgVertexShader(int guid,VertexBindingFormat format,unsigned int handle)
  {
    mGuid   = guid;
    mFormat = format;
    mHandle = handle;
  }

  int  GetGuid(void) const { return mGuid; };

  void AddBinding(VertexBindingType type,unsigned int bind_handle)
  {
    CgVertexBind b(type,bind_handle);
    mBindings.push_back(b);
  }

  bool Activate(void);

  bool ActivateBindings(CgBinding *bindings,void *context1,void *context2);

private:
  int                 mGuid;      /* Our Guid Id number */
  unsigned int        mHandle;    /* Handle to our CgShader program. */
  VertexBindingFormat mFormat;    /* The format our vertex data is in. */
  CgVertexBindVector  mBindings;  /* STL vector of all of our vertex binding points. */
};

typedef std::map< int , CgVertexShader * > CgVertexShaderMap;


/**
   @class CgPixelBind

   @brief A single bind point for a pixel  CgProgram

   This class manages a single binding point between a particular instance of a
   Cg pixel shader fragment and remaps it to a logical VertexBindingType.
   The application must be able to provide the data that is needed for this
   bind point.  For example the world*view*projection matrix.

   @see enum PixelBindingType

   @warning To minimize dependencies the internal bind point handle is case to a generic unsigned int

   @author John W. Ratcliff

   @date  July 18, 2002

*/
class CgPixelBind
{
public:
  CgPixelBind(void) { };

  CgPixelBind(PixelBindingType b,unsigned int handle)
  {
    mBind = b;
    mHandle = handle;
  };

  bool Activate(CgBinding *bindings,unsigned int program_handle,void *context1,void *context2);

private:
  PixelBindingType mBind;     /* Type of bind this is. */
  unsigned int      mHandle;  /* Cg bind iteration handle. */
};

// STL vector of binding points
typedef std::vector< CgPixelBind > CgPixelBindVector;


/**
   @class CgPixelShader

   @brief A single instance of a PixelShader

   This class represents a single instance of a pixel shader program.

   @warning To minimize dependencies the internal Cg program handle is case to an unsigned int

   @author John W. Ratcliff

   @date  July 18, 2002

*/
class CgPixelShader
{
public:

  CgPixelShader(int guid,unsigned int handle)
  {
    mGuid   = guid;
    mHandle = handle;
  }

  int  GetGuid(void) const { return mGuid; }; /* Return our assigned Guid */

  void AddBinding(PixelBindingType type,unsigned int bind_handle) /* Add a pixel binding point */
  {
    CgPixelBind b(type,bind_handle);
    mBindings.push_back(b);
  }

  bool Activate(void);

  bool ActivateBindings(CgBinding *bindings,void *context1,void *context2);

private:
  int                 mGuid;    /* Our Id numer. */
  unsigned int        mHandle;  /* Handle to our CgShader program. */
  CgPixelBindVector   mBindings; /* STL vector of our binding points */
};

typedef std::map< int , CgPixelShader * > CgPixelShaderMap;

/**
   @class CgBinding

   @brief Virtual base class inherited by application to provide run-time binding with the shader program.

   The application needs to only provide two services to run vertex shader programs.  First it must be
   able to validate whether or not it knows how to manage a particular VertexBindingType and be able to
   provide real-time data for those binding types it says it can support.  The same thing is true to bind
   pixel shader programs as well.

   @see enum  VertexBindingFormat
   @see enum  VertexBindingType
   @see enum  PixelBindingType

   @author John W. Ratcliff

   @date  July 18, 2002

*/
class CgBinding
{
public:

  CgBinding(void);
  ~CgBinding(void);


  // validate that our application knows how to handle arguments of this type.
  // Pure virtual function, application must provided this service
  virtual bool ValidateVertexBindingType(VertexBindingType type) =0;

  virtual void * GetVertexBindingType(VertexBindingType type,       // keyword we need the results for
                                      void *context1,               // up to 2 contexts for this shader
                                      void *context2) = 0;          //

  // validate that our application knows how to handle arguments of this type.
  // Pure virtual function, application must provided this service
  virtual bool ValidatePixelBindingType(VertexBindingType type) =0;

  virtual void * GetPixelBindingType(PixelBindingType type,       // keyword we need the results for
                                     void *context1,               // up to 2 contexts for this shader
                                     void *context2) = 0;          //


  // convenience function, converts bind name, including all possible aliases to logical vertexbindingtype keyword.
  VertexBindingType AsciiToVertexBindingType(const char *bind_name);
  PixelBindingType  AsciiToPixelBindingType(const char *bind_name);

  CgVertexShader * CreateVertexShader(const char *program,VertexBindingFormat type,unsigned int device);
  CgPixelShader  * CreatePixelShader(const char *program,unsigned int device);

  bool       ReleaseVertexShader(CgVertexShader *shader); // this shader is toast.
  bool       ReleasePixelShader(CgPixelShader *shader); // this shader is toast.

  void * GetVertexAttributes(VertexBindingFormat type);

  const char * GetLastListing(void) const { return mLastListing; }; /* Return last compilation output */
  const char * GetLastError(void)   const { return mLastError; };   /* Return reason for last error. */

private:
  int                 mGuid;                  /* Just a guid we keep around to keep track of shaders we allocate. */
  void               *mContextContainer;      /* Cg container class for all shaders loaded. */
  CgVertexShaderMap   mVertexShaders;         /* STL map, indexed by Guid, of all loaded vertex shaders */
  CgPixelShaderMap    mPixelShaders;          /* STL map, indexed by Guid, of all loaded pixel shaders */
  const char         *mLastListing;           /* Pointer to last compilation output. */
  char                mLastError[1024];       /* Our reason we failed when attempting to load a shader */
};


#endif
